@@ -2,6 +2,8 @@
 # Copyright (c) Microsoft Corporation. All rights reserved.
 # Licensed under the MIT License. See License.txt in the project root for license information.
 # --------------------------------------------------------------------------------------------
+# pylint: disable=line-too-long
+
 from collections import OrderedDict
 import sys
 import os
@@ -11,13 +13,13 @@ import zipfile
 import traceback
 import hashlib
 from subprocess import check_output, STDOUT, CalledProcessError
-from six.moves.urllib.parse import urlparse  # pylint: disable=import-error
+from urllib.parse import urlparse
 
-import requests
 from pkg_resources import parse_version
 
+from azure.cli.core import CommandIndex
 from azure.cli.core.util import CLIError, reload_module
-from azure.cli.core.extension import (extension_exists, get_extension_path, get_extensions, get_extension_modname,
+from azure.cli.core.extension import (extension_exists, build_extension_path, get_extensions, get_extension_modname,
                                       get_extension, ext_compat_with_cli,
                                       EXT_METADATA_ISPREVIEW, EXT_METADATA_ISEXPERIMENTAL,
                                       WheelExtension, DevExtension, ExtensionNotInstalledException, WHEEL_INFO_RE)
@@ -37,13 +39,14 @@ OUT_KEY_TYPE = 'extensionType'
 OUT_KEY_METADATA = 'metadata'
 OUT_KEY_PREVIEW = 'preview'
 OUT_KEY_EXPERIMENTAL = 'experimental'
+OUT_KEY_PATH = 'path'
 
 IS_WINDOWS = sys.platform.lower() in ['windows', 'win32']
 LIST_FILE_PATH = os.path.join(os.sep, 'etc', 'apt', 'sources.list.d', 'azure-cli.list')
 LSB_RELEASE_FILE = os.path.join(os.sep, 'etc', 'lsb-release')
 
 
-def _run_pip(pip_exec_args):
+def _run_pip(pip_exec_args, extension_path=None):
     cmd = [sys.executable, '-m', 'pip'] + pip_exec_args + ['-vv', '--disable-pip-version-check', '--no-cache-dir']
     logger.debug('Running: %s', cmd)
     try:
@@ -53,11 +56,14 @@ def _run_pip(pip_exec_args):
     except CalledProcessError as e:
         logger.debug(e.output)
         logger.debug(e)
+        if "PermissionError: [WinError 5]" in e.output:
+            logger.warning("You do not have the permission to add extensions in the target directory%s. You may need to rerun on a shell as administrator.", ': ' + os.path.split(extension_path)[0] if extension_path else '')
         returncode = e.returncode
     return returncode
 
 
 def _whl_download_from_url(url_parse_result, ext_file):
+    import requests
     from azure.cli.core.util import should_disable_connection_verify
     url = url_parse_result.geturl()
     r = requests.get(url, stream=True, verify=(not should_disable_connection_verify()))
@@ -79,8 +85,19 @@ def _validate_whl_extension(ext_file):
     check_version_compatibility(azext_metadata)
 
 
-def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_proxy=None):  # pylint: disable=too-many-statements
-    cmd.cli_ctx.get_progress_controller().add(message='Analyzing')
+def _get_extension_info_from_source(source):
+    url_parse_result = urlparse(source)
+    is_url = (url_parse_result.scheme == 'http' or url_parse_result.scheme == 'https')
+    whl_filename = os.path.basename(url_parse_result.path) if is_url else os.path.basename(source)
+    parsed_filename = WHEEL_INFO_RE(whl_filename)
+    # Extension names can have - but .whl format changes it to _ (PEP 0427). Undo this.
+    extension_name = parsed_filename.groupdict().get('name').replace('_', '-') if parsed_filename else None
+    extension_version = parsed_filename.groupdict().get('ver') if parsed_filename else None
+    return extension_name, extension_version
+
+
+def _add_whl_ext(cli_ctx, source, ext_sha256=None, pip_extra_index_urls=None, pip_proxy=None, system=None):  # pylint: disable=too-many-statements
+    cli_ctx.get_progress_controller().add(message='Analyzing')
     if not source.endswith('.whl'):
         raise ValueError('Unknown extension type. Only Python wheels are supported.')
     url_parse_result = urlparse(source)
@@ -94,14 +111,17 @@ def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_pr
         raise CLIError('Unable to determine extension name from {}. Is the file name correct?'.format(source))
     if extension_exists(extension_name, ext_type=WheelExtension):
         raise CLIError('The extension {} already exists.'.format(extension_name))
+    if extension_name == 'rdbms-connect':
+        _install_deps_for_psycopg2()
     ext_file = None
     if is_url:
         # Download from URL
         tmp_dir = tempfile.mkdtemp()
         ext_file = os.path.join(tmp_dir, whl_filename)
         logger.debug('Downloading %s to %s', source, ext_file)
+        import requests
         try:
-            cmd.cli_ctx.get_progress_controller().add(message='Downloading')
+            cli_ctx.get_progress_controller().add(message='Downloading')
             _whl_download_from_url(url_parse_result, ext_file)
         except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError) as err:
             raise CLIError('Please ensure you have network connection. Error detail: {}'.format(str(err)))
@@ -123,7 +143,7 @@ def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_pr
             raise CLIError("The checksum of the extension does not match the expected value. "
                            "Use --debug for more information.")
     try:
-        cmd.cli_ctx.get_progress_controller().add(message='Validating')
+        cli_ctx.get_progress_controller().add(message='Validating')
         _validate_whl_extension(ext_file)
     except AssertionError:
         logger.debug(traceback.format_exc())
@@ -133,9 +153,9 @@ def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_pr
     logger.debug('Validation successful on %s', ext_file)
     # Check for distro consistency
     check_distro_consistency()
-    cmd.cli_ctx.get_progress_controller().add(message='Installing')
+    cli_ctx.get_progress_controller().add(message='Installing')
     # Install with pip
-    extension_path = get_extension_path(extension_name)
+    extension_path = build_extension_path(extension_name, system)
     pip_args = ['install', '--target', extension_path, ext_file]
 
     if pip_proxy:
@@ -146,7 +166,7 @@ def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_pr
 
     logger.debug('Executing pip with args: %s', pip_args)
     with HomebrewPipPatch():
-        pip_status_code = _run_pip(pip_args)
+        pip_status_code = _run_pip(pip_args, extension_path)
     if pip_status_code > 0:
         logger.debug('Pip failed so deleting anything we might have installed at %s', extension_path)
         shutil.rmtree(extension_path, ignore_errors=True)
@@ -160,6 +180,75 @@ def _add_whl_ext(cmd, source, ext_sha256=None, pip_extra_index_urls=None, pip_pr
     return extension_name
 
 
+def _install_deps_for_psycopg2():  # pylint: disable=too-many-statements
+    # Below system dependencies are required to install the psycopg2 dependency for Linux and macOS
+    import platform
+    import subprocess
+    from azure.cli.core.util import get_linux_distro
+    from azure.cli.core._environment import _ENV_AZ_INSTALLER
+    installer = os.getenv(_ENV_AZ_INSTALLER)
+    system = platform.system()
+    if system == 'Darwin':
+        subprocess.call(['xcode-select', '--install'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+        if installer != 'HOMEBREW':
+            from shutil import which
+            if which('brew') is None:
+                logger.warning('You may need to install postgresql with homebrew first before you install this extension.')
+                return
+        exit_code = subprocess.call(['brew', 'list', 'postgresql'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+        if exit_code != 0:
+            update_cmd = ['brew', 'install', 'postgresql']
+            logger.warning('This extension depends on postgresql and it will be installed first.')
+            logger.debug("Install dependencies with '%s'", " ".join(update_cmd))
+            subprocess.call(update_cmd)
+        # Fix the issue of -lssl not found during building psycopg2
+        if os.environ.get('LIBRARY_PATH') is None:
+            os.environ['LIBRARY_PATH'] = '/usr/local/opt/openssl/lib/'
+        else:
+            os.environ['LIBRARY_PATH'] = os.pathsep.join([
+                os.environ.get('LIBRARY_PATH'),
+                '/usr/local/opt/openssl/lib/'
+            ])
+    elif system == 'Linux':
+        distname, _ = get_linux_distro()
+        distname = distname.lower().strip()
+        if installer == 'DEB' or any(x in distname for x in ['ubuntu', 'debian']):
+            from azure.cli.core.util import in_cloud_console
+            if in_cloud_console():
+                raise CLIError("This extension is not supported in Cloud Shell as you do not have permission to install extra dependencies.")
+            exit_code = subprocess.call(['dpkg', '-s', 'gcc', 'libpq-dev', 'python3-dev'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
+            if exit_code != 0:
+                logger.warning('This extension depends on gcc, libpq-dev, python3-dev and they will be installed first.')
+                apt_update_cmd = 'apt-get update'.split()
+                apt_install_cmd = 'apt-get install -y gcc libpq-dev python3-dev'.split()
+                if os.geteuid() != 0:  # pylint: disable=no-member
+                    apt_update_cmd.insert(0, 'sudo')
+                    apt_install_cmd.insert(0, 'sudo')
+                exit_code = subprocess.call(apt_update_cmd, True)
+                if exit_code == 0:
+                    logger.debug("Install dependencies with '%s'", " ".join(apt_install_cmd))
+                    subprocess.call(apt_install_cmd, True)
+        elif installer == 'RPM' or any(x in distname for x in ['centos', 'rhel', 'red hat', 'fedora', 'opensuse', 'suse', 'sles']):
+            if any(x in distname for x in ['centos', 'rhel', 'red hat', 'fedora']):
+                yum_install_cmd = 'yum install -y gcc postgresql-devel python3-devel'.split()
+                if os.geteuid() != 0:  # pylint: disable=no-member
+                    yum_install_cmd.insert(0, 'sudo')
+                logger.debug("Install dependencies with '%s'", " ".join(yum_install_cmd))
+                logger.warning('This extension depends on gcc, postgresql-devel, python3-devel and they will be installed first if not exist.')
+                subprocess.call(yum_install_cmd)
+            elif any(x in distname for x in ['opensuse', 'suse', 'sles']):
+                zypper_refresh_cmd = ['zypper', 'refresh']
+                zypper_install_cmd = 'zypper install -y gcc postgresql-devel python3-devel'.split()
+                logger.warning('This extension depends on gcc postgresql-devel, python3-devel and they will be installed first if not exist.')
+                if os.geteuid() != 0:  # pylint: disable=no-member
+                    zypper_refresh_cmd.insert(0, 'sudo')
+                    zypper_install_cmd.insert(0, 'sudo')
+                exit_code = subprocess.call(zypper_refresh_cmd)
+                if exit_code == 0:
+                    logger.debug("Install dependencies with '%s'", " ".join(zypper_install_cmd))
+                    subprocess.call(zypper_install_cmd)
+
+
 def is_valid_sha256sum(a_file, expected_sum):
     sha256 = hashlib.sha256()
     with open(a_file, 'rb') as f:
@@ -168,12 +257,12 @@ def is_valid_sha256sum(a_file, expected_sum):
     return expected_sum == computed_hash, computed_hash
 
 
-def _augment_telemetry_with_ext_info(extension_name):
+def _augment_telemetry_with_ext_info(extension_name, ext=None):
     # The extension must be available before calling this otherwise we can't get the version from metadata
     if not extension_name:
         return
     try:
-        ext = get_extension(extension_name)
+        ext = ext or get_extension(extension_name)
         ext_version = ext.version
         set_extension_management_detail(extension_name, ext_version)
     except Exception:  # nopa pylint: disable=broad-except
@@ -182,52 +271,81 @@ def _augment_telemetry_with_ext_info(extension_name):
 
 
 def check_version_compatibility(azext_metadata):
-    is_compatible, cli_core_version, min_required, max_required = ext_compat_with_cli(azext_metadata)
-    logger.debug("Extension compatibility result: is_compatible=%s cli_core_version=%s min_required=%s "
-                 "max_required=%s", is_compatible, cli_core_version, min_required, max_required)
+    is_compatible, cli_core_version, min_required, max_required, min_ext_required = ext_compat_with_cli(azext_metadata)
+    # logger.debug("Extension compatibility result: is_compatible=%s cli_core_version=%s min_required=%s "
+    #              "max_required=%s", is_compatible, cli_core_version, min_required, max_required)
     if not is_compatible:
-        min_max_msg_fmt = "The '{}' extension is not compatible with this version of the CLI.\n" \
-                          "You have CLI core version {} and this extension " \
-                          "requires ".format(azext_metadata.get('name'), cli_core_version)
-        if min_required and max_required:
-            min_max_msg_fmt += 'a min of {} and max of {}.'.format(min_required, max_required)
+        ext_name = azext_metadata.get('name')
+        ext_version = azext_metadata.get('version')
+        min_max_msgs = [
+            f"The '{ext_name}' extension version {ext_version} is not compatible with your current CLI core version {cli_core_version}."
+        ]
+        if min_ext_required:
+            min_max_msgs.append(f"This CLI core requires a min of {min_ext_required} for the '{ext_name}' extension.")
+            min_max_msgs.append(f"Please run 'az extension update -n {ext_name}' to update it.")
+        elif min_required and max_required:
+            min_max_msgs.append(f'This extension requires a min of {min_required} and max of {max_required} CLI core.')
+            min_max_msgs.append("Please run 'az upgrade' to upgrade to a compatible version.")
         elif min_required:
-            min_max_msg_fmt += 'a min of {}.'.format(min_required)
+            min_max_msgs.append(f'This extension requires a min of {min_required} CLI core.')
+            min_max_msgs.append("Please run 'az upgrade' to upgrade to a compatible version.")
         elif max_required:
-            min_max_msg_fmt += 'a max of {}.'.format(max_required)
-        min_max_msg_fmt += '\nPlease install a compatible extension version or remove it.'
-        raise CLIError(min_max_msg_fmt)
+            min_max_msgs.append(f'This extension requires a max of {max_required} CLI core.')
+            # we do not want users to downgrade CLI core version, so we suggest updating the extension in this case
+            min_max_msgs.append(f"Please run 'az extension update -n {ext_name}' to update the extension.")
+
+        raise CLIError("\n".join(min_max_msgs))
 
 
-def add_extension(cmd, source=None, extension_name=None, index_url=None, yes=None,  # pylint: disable=unused-argument
-                  pip_extra_index_urls=None, pip_proxy=None):
+def add_extension(cmd=None, source=None, extension_name=None, index_url=None, yes=None,  # pylint: disable=unused-argument, too-many-statements
+                  pip_extra_index_urls=None, pip_proxy=None, system=None,
+                  version=None, cli_ctx=None, upgrade=None):
     ext_sha256 = None
+
+    version = None if version == 'latest' else version
+    cmd_cli_ctx = cli_ctx or cmd.cli_ctx
     if extension_name:
-        cmd.cli_ctx.get_progress_controller().add(message='Searching')
+        cmd_cli_ctx.get_progress_controller().add(message='Searching')
         ext = None
+        set_extension_management_detail(extension_name, version)
         try:
             ext = get_extension(extension_name)
         except ExtensionNotInstalledException:
             pass
         if ext:
             if isinstance(ext, WheelExtension):
-                logger.warning("Extension '%s' is already installed.", extension_name)
+                if not upgrade:
+                    logger.warning("Extension '%s' is already installed.", extension_name)
+                    return
+                logger.warning("Extension '%s' %s is already installed.", extension_name, ext.get_version())
+                if version and version == ext.get_version():
+                    return
+                logger.warning("It will be overridden with version {}.".format(version) if version else "It will be updated if available.")
+                update_extension(cmd=cmd, extension_name=extension_name, index_url=index_url, pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy, cli_ctx=cli_ctx, version=version)
                 return
             logger.warning("Overriding development version of '%s' with production version.", extension_name)
         try:
-            source, ext_sha256 = resolve_from_index(extension_name, index_url=index_url)
+            source, ext_sha256 = resolve_from_index(extension_name, index_url=index_url, target_version=version, cli_ctx=cmd_cli_ctx)
         except NoExtensionCandidatesError as err:
             logger.debug(err)
-            raise CLIError("No matching extensions for '{}'. Use --debug for more information.".format(extension_name))
-    extension_name = _add_whl_ext(cmd=cmd, source=source, ext_sha256=ext_sha256,
-                                  pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy)
-    _augment_telemetry_with_ext_info(extension_name)
+
+            if version:
+                err = "No matching extensions for '{} ({})'. Use --debug for more information.".format(extension_name, version)
+            else:
+                err = "No matching extensions for '{}'. Use --debug for more information.".format(extension_name)
+            raise CLIError(err)
+    ext_name, ext_version = _get_extension_info_from_source(source)
+    set_extension_management_detail(extension_name if extension_name else ext_name, ext_version)
+    extension_name = _add_whl_ext(cli_ctx=cmd_cli_ctx, source=source, ext_sha256=ext_sha256,
+                                  pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy, system=system)
     try:
-        if extension_name and get_extension(extension_name).experimental:
+        ext = get_extension(extension_name)
+        if extension_name and ext.experimental:
             logger.warning("The installed extension '%s' is experimental and not covered by customer support. "
                            "Please use with discretion.", extension_name)
-        elif extension_name and get_extension(extension_name).preview:
+        elif extension_name and ext.preview:
             logger.warning("The installed extension '%s' is in preview.", extension_name)
+        CommandIndex().invalidate()
     except ExtensionNotInstalledException:
         pass
 
@@ -245,15 +363,16 @@ def remove_extension(extension_name):
                 "Extension '{name}' was installed in development mode. Remove using "
                 "`azdev extension remove {name}`".format(name=extension_name))
         # We call this just before we remove the extension so we can get the metadata before it is gone
-        _augment_telemetry_with_ext_info(extension_name)
-        shutil.rmtree(get_extension_path(extension_name), onerror=log_err)
+        _augment_telemetry_with_ext_info(extension_name, ext)
+        shutil.rmtree(ext.path, onerror=log_err)
+        CommandIndex().invalidate()
     except ExtensionNotInstalledException as e:
         raise CLIError(e)
 
 
 def list_extensions():
     return [{OUT_KEY_NAME: ext.name, OUT_KEY_VERSION: ext.version, OUT_KEY_TYPE: ext.ext_type,
-             OUT_KEY_PREVIEW: ext.preview, OUT_KEY_EXPERIMENTAL: ext.experimental}
+             OUT_KEY_PREVIEW: ext.preview, OUT_KEY_EXPERIMENTAL: ext.experimental, OUT_KEY_PATH: ext.path}
             for ext in get_extensions()]
 
 
@@ -263,47 +382,52 @@ def show_extension(extension_name):
         return {OUT_KEY_NAME: extension.name,
                 OUT_KEY_VERSION: extension.version,
                 OUT_KEY_TYPE: extension.ext_type,
-                OUT_KEY_METADATA: extension.metadata}
+                OUT_KEY_METADATA: extension.metadata,
+                OUT_KEY_PATH: extension.path}
     except ExtensionNotInstalledException as e:
         raise CLIError(e)
 
 
-def update_extension(cmd, extension_name, index_url=None, pip_extra_index_urls=None, pip_proxy=None):
+def update_extension(cmd=None, extension_name=None, index_url=None, pip_extra_index_urls=None, pip_proxy=None, cli_ctx=None, version=None):
     try:
+        cmd_cli_ctx = cli_ctx or cmd.cli_ctx
         ext = get_extension(extension_name, ext_type=WheelExtension)
         cur_version = ext.get_version()
         try:
-            download_url, ext_sha256 = resolve_from_index(extension_name, cur_version=cur_version, index_url=index_url)
+            download_url, ext_sha256 = resolve_from_index(extension_name, cur_version=cur_version, index_url=index_url, target_version=version, cli_ctx=cmd_cli_ctx)
+            _, ext_version = _get_extension_info_from_source(download_url)
+            set_extension_management_detail(extension_name, ext_version)
         except NoExtensionCandidatesError as err:
             logger.debug(err)
-            raise CLIError("No updates available for '{}'. Use --debug for more information.".format(extension_name))
+            msg = "Extension {} with version {} not found.".format(extension_name, version) if version else "No updates available for '{}'. Use --debug for more information.".format(extension_name)
+            logger.warning(msg)
+            return
         # Copy current version of extension to tmp directory in case we need to restore it after a failed install.
         backup_dir = os.path.join(tempfile.mkdtemp(), extension_name)
-        extension_path = get_extension_path(extension_name)
+        extension_path = ext.path
         logger.debug('Backing up the current extension: %s to %s', extension_path, backup_dir)
         shutil.copytree(extension_path, backup_dir)
         # Remove current version of the extension
         shutil.rmtree(extension_path)
         # Install newer version
         try:
-            _add_whl_ext(cmd=cmd, source=download_url, ext_sha256=ext_sha256,
+            _add_whl_ext(cli_ctx=cmd_cli_ctx, source=download_url, ext_sha256=ext_sha256,
                          pip_extra_index_urls=pip_extra_index_urls, pip_proxy=pip_proxy)
             logger.debug('Deleting backup of old extension at %s', backup_dir)
             shutil.rmtree(backup_dir)
-            # This gets the metadata for the extension *after* the update
-            _augment_telemetry_with_ext_info(extension_name)
         except Exception as err:
             logger.error('An error occurred whilst updating.')
             logger.error(err)
             logger.debug('Copying %s to %s', backup_dir, extension_path)
             shutil.copytree(backup_dir, extension_path)
             raise CLIError('Failed to update. Rolled {} back to {}.'.format(extension_name, cur_version))
+        CommandIndex().invalidate()
     except ExtensionNotInstalledException as e:
         raise CLIError(e)
 
 
-def list_available_extensions(index_url=None, show_details=False):
-    index_data = get_index_extensions(index_url=index_url)
+def list_available_extensions(index_url=None, show_details=False, cli_ctx=None):
+    index_data = get_index_extensions(index_url=index_url, cli_ctx=cli_ctx)
     if show_details:
         return index_data
     installed_extensions = get_extensions(ext_type=WheelExtension)
@@ -333,6 +457,45 @@ def list_available_extensions(index_url=None, show_details=False):
     return results
 
 
+def list_versions(extension_name, index_url=None, cli_ctx=None):
+    index_data = get_index_extensions(index_url=index_url, cli_ctx=cli_ctx)
+
+    try:
+        exts = index_data[extension_name]
+    except Exception:
+        raise CLIError('Extension {} not found.'.format(extension_name))
+
+    try:
+        installed_ext = get_extension(extension_name, ext_type=WheelExtension)
+    except ExtensionNotInstalledException:
+        installed_ext = None
+
+    results = []
+    latest_compatible_version = None
+
+    for ext in sorted(exts, key=lambda c: parse_version(c['metadata']['version']), reverse=True):
+        compatible = ext_compat_with_cli(ext['metadata'])[0]
+        ext_version = ext['metadata']['version']
+        if latest_compatible_version is None and compatible:
+            latest_compatible_version = ext_version
+        installed = ext_version == installed_ext.version if installed_ext else False
+        if installed and parse_version(latest_compatible_version) > parse_version(installed_ext.version):
+            installed = str(True) + ' (upgrade available)'
+        version = ext['metadata']['version']
+        if latest_compatible_version == ext_version:
+            version = version + ' (max compatible version)'
+        results.append({
+            'name': extension_name,
+            'version': version,
+            'preview': ext['metadata'].get(EXT_METADATA_ISPREVIEW, False),
+            'experimental': ext['metadata'].get(EXT_METADATA_ISEXPERIMENTAL, False),
+            'installed': installed,
+            'compatible': compatible
+        })
+    results.reverse()
+    return results
+
+
 def reload_extension(extension_name, extension_module=None):
     return reload_module(extension_module if extension_module else get_extension_modname(ext_name=extension_name))
 
@@ -340,6 +503,24 @@ def reload_extension(extension_name, extension_module=None):
 def add_extension_to_path(extension_name, ext_dir=None):
     ext_dir = ext_dir or get_extension(extension_name).path
     sys.path.append(ext_dir)
+    # If this path update should have made a new "azure" module available,
+    # extend the existing module with its path. This allows extensions to
+    # include (or depend on) Azure SDK modules that are not yet part of
+    # the CLI. This applies to both the "azure" and "azure.mgmt" namespaces,
+    # but ensures that modules installed by the CLI take priority.
+    azure_dir = os.path.join(ext_dir, "azure")
+    if os.path.isdir(azure_dir):
+        import azure
+        azure.__path__.append(azure_dir)
+        azure_mgmt_dir = os.path.join(azure_dir, "mgmt")
+        if os.path.isdir(azure_mgmt_dir):
+            try:
+                # Should have been imported already, so this will be quick
+                import azure.mgmt
+            except ImportError:
+                pass
+            else:
+                azure.mgmt.__path__.append(azure_mgmt_dir)
 
 
 def get_lsb_release():
